/* Fallout game, Kavaklis Yannis 11/93 */
#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include "xfallout.h"

#define offset(field) XtOffset(struct resource_struct *, field)
#define soffset(field) XtOffset(struct shape_table *, field)
#define STANDARD_SPEED  10


struct resource_struct
{
  Boolean usescorefile;
} resources;

struct shape_table {
	Pixel           foreground, background;
}               shape_table[7];




static XtResource ShapeResources[] = {
	{"foreground", "Foreground", XtRPixel, sizeof(Pixel),
	soffset(foreground), XtRString, XtDefaultForeground},
	{"background", "Background", XtRPixel, sizeof(Pixel),
	soffset(background), XtRString, XtDefaultBackground},
};

static XtResource Resources[] = {
{"useScoreFile",      "Boolean",      XtRBoolean,     sizeof(Boolean),
     offset(usescorefile),      XtRImmediate,   (XtPointer)True},
};

#include <X11/Xaw/Text.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Box.h>
#include "bit.xbm"
#include "dust.xbm"
#include "block.xbm"
#include "zet.xbm"
#include "bomb.xbm"
#include "f.xbm"
#include "a.xbm"
#include "l.xbm"
#include "o.xbm"
#include "u.xbm"
#include "t.xbm"


int             game_grid[UWIDTH][UHEIGHT];
int             tmp_grid[UWIDTH][UHEIGHT];
int             removed;
Pixmap          bit, DustPixmap, block, ZetaPixmap, BombPixmap;
Pixmap		letters[6];
int             visible_zeta;
int             visible_bomb;
int             Bombs_number = 0;
GC              BlcksGC[7];
char            BlockSet[3];
char            NextBlockSet[3];
char            BlockOrient;
int             BlockXpos, BlockYpos;
int             playing = 0;
int             start_level = 0;
int             next = 1;
int             ZetaXpos = 0;
int             ZetaYpos = 0;
int             BombXpos = 0;
int             BombYpos = 0;
int             color_screen;
int             usescorefile = 1;

extern void     quit_proc();
extern void     pause_proc();
extern void     resume_proc();
extern void     left_proc();
extern void     rot_proc();
extern void     drop_proc();
extern void     circ_proc();
extern void     right_proc();
extern void     new_game_proc();
extern void     restore_widget();
extern void     activate_bomb_proc();
extern void     increase_level_proc();
extern void     decrease_level_proc();
extern void     next_proc();

int             can_down();
static XtActionsRec actions[] = {
	{"Refresh", restore_widget},
	{"Quit", quit_proc},
	{"RotateCW", rot_proc},
	{"Circulate", circ_proc},
	{"MoveLeft", left_proc},
	{"MoveRight", right_proc},
	{"Pause", pause_proc},
	{"Resume", resume_proc},
	{"New", new_game_proc},
	{"Bomb", activate_bomb_proc},
	{"Next", next_proc},
	{"Level", increase_level_proc},
	{"UnLevel", decrease_level_proc},
	{"Drop", drop_proc}
};

static XrmOptionDescRec Options[] = {
	{"-score", "useScoreFile", XrmoptionNoArg, "TRUE"},
	{"-noscore", "useScoreFile", XrmoptionNoArg, "FALSE"},
};

XtAppContext    context;
Widget          toplevel, canvas, frame, tbar;
Display        *display;
int             screenno;
GC              lineGC,thickGC;
int             score;

main(argc, argv, envp)
	int             argc;
	char          **argv;
	char          **envp;
{

	int             i, j,n;
	XrmDatabase db;
	XVisualInfo vi;
	static XtConvertArgRec screenConvertArg[] = {
		{XtWidgetBaseOffset, (caddr_t) XtOffset(Widget, core.screen),
		sizeof(Screen *)}
	};

	XGCValues       gcvalues;
	unsigned long   valuemask;
	Arg		args[20];
	Dimension   width, height;

	XtToolkitInitialize();
	context = XtCreateApplicationContext();
	display = XtOpenDisplay(context, NULL, NULL, "Fallout", Options, XtNumber(Options), &argc, argv);
	if (!display)
                {
                fprintf(stderr,"Unable to connect to display, exiting.\n");
                exit(-1);
                }

	screenno = DefaultScreen(display);
	/*decide if we are on a monochrome screen */
	if (XMatchVisualInfo( display, screenno, 4, PseudoColor, &vi )
      || XMatchVisualInfo( display, screenno, 4, StaticColor, &vi )
      || XMatchVisualInfo( display, screenno, 6, DirectColor, &vi )
      || XMatchVisualInfo( display, screenno, 8, DirectColor, &vi )
      || XMatchVisualInfo( display, screenno, 6, TrueColor, &vi ))
  {
	color_screen = 1;
  }
  else
  {
	color_screen = 0;
  }
	toplevel = XtVaAppCreateShell(NULL, "Fallout",
				      applicationShellWidgetClass,
				      display,
				      NULL);

	XtGetApplicationResources( toplevel, (caddr_t) &resources,
                            Resources, XtNumber(Resources),
                            NULL, (Cardinal) 0);
	XtAppAddActions(context, actions, XtNumber(actions));
	XtAppAddConverter(context, "String", "Bitmap", XmuCvtStringToBitmap,
			  screenConvertArg, XtNumber(screenConvertArg));

	for (i = 0; i < 7; i++) {
		static char    *names[7] = {"object0", "object1", "object2", "object3", "object4",
		"object5", "object6"};

		XtGetSubresources(toplevel, (caddr_t) & shape_table[i], names[i], "Object",
				  ShapeResources, XtNumber(ShapeResources),
				  NULL, (Cardinal) 0);
	}
	frame = XtVaCreateManagedWidget("Frame", formWidgetClass, toplevel, NULL);
	canvas = XtVaCreateManagedWidget("Canvas", simpleWidgetClass, frame,
		    XtNwidth, (XtArgVal) (UWIDTH * BLCK_SIZE + 300),
			XtNheight, (XtArgVal) (UHEIGHT * BLCK_SIZE),
					 NULL);
	XtInstallAllAccelerators(canvas, toplevel);
	XtRealizeWidget(toplevel);
	XtMapWidget(toplevel);

	usescorefile = (resources.usescorefile == True);
	gcvalues.function = GXcopy;
	gcvalues.foreground = WhitePixel(display, 0);
	gcvalues.background = BlackPixel(display, 0);
	valuemask = GCFunction | GCForeground | GCBackground;
	lineGC = XCreateGC(display, XtWindow(canvas), valuemask, &gcvalues);
	gcvalues.line_width=6;
	valuemask|=GCLineWidth;
	thickGC = XCreateGC(display, XtWindow(canvas), valuemask, &gcvalues);
	XDrawLine(display, XtWindow(canvas), lineGC, UWIDTH*BLCK_SIZE+1, 
			0, UWIDTH*BLCK_SIZE+1, UHEIGHT* BLCK_SIZE);
	init_all();

	n=0;
        XtSetArg(args[n],XtNwidth,&width); n++;
        XtSetArg(args[n],XtNheight,&height); n++;
	XtGetValues(toplevel,args,n);

	n=0;
        XtSetArg(args[n],XtNminWidth,width); n++;
        XtSetArg(args[n],XtNmaxWidth,width); n++;
        XtSetArg(args[n],XtNminHeight,height); n++;
        XtSetArg(args[n],XtNmaxHeight,height); n++;
        XtSetValues(toplevel,args,n);

	XtAppMainLoop(context);
}

static int paused = 0;

void
resume_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	if (playing && paused) {
		start_timer();
		paused = 0;
	}
}
void
pause_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	if (playing) {
		stop_timer();
		paused =1;
	}
}
void
quit_proc()
{
	exit(1);
}



void
calculate()
{
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;
	int             i, j;

	if (BlockOrient == VERTICAL) {
		if (visible_zeta && (x != ZetaXpos))
			clear_zeta();
		if (visible_bomb) {
		if ((x != BombXpos))
			clear_bomb();
		else
			add_bomb();
		}
		game_grid[x][y - 1] = BlockSet[0] + 1;
		game_grid[x][y] = BlockSet[1] + 1;
		game_grid[x][y + 1] = BlockSet[2] + 1;
	} else {
		clear_shape();
		if (visible_zeta && ((x - 1) > ZetaXpos || x + 1 < ZetaXpos))
			clear_zeta();
		if(visible_bomb) {
		if (((x - 1) > BombXpos || x + 1 < BombXpos))
			clear_bomb();
		else
			add_bomb();
		}
		for (i = -1; i < 2; i++) {
			j = 0;
			while (((y + j) < (UHEIGHT - 1)) && (game_grid[x + i][y + j + 1] == 0))
				j++;
			draw_bit(BlockSet[i + 1],(x + i),(y + j));
			game_grid[x + i][y + j] = BlockSet[i + 1] + 1;
		}
	}
	do  {
		removed = 0;
		find_shapes();
		XFlush(XtDisplay(canvas));
		usleep(500000);
		if (removed) {
			update_positions();
		}
		update_info();
	}while (removed);
}


update_positions()
{
int i,j;
for (i = 0; i < UWIDTH; i++)
 for (j = 0; j < UHEIGHT; j++)
    if (tmp_grid[i][j]) {
      int             k;
      score += 10;
      for (k = j; k > 0; k--) {
       game_grid[i][k] = game_grid[i][k - 1];
       if (game_grid[i][k])
        draw_bit(game_grid[i][k] - 1,i,k);
       else
        clear_bit(i,k);
      }
     }
}

void
drop_block(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	static int      i = 0;
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;


	if (can_down()) {
		clear_shape();
		BlockYpos += BLCK_SIZE;
		draw_shape();
	} else {
		if (y == 1) {
			stop_timer();
			finish();
			if (usescorefile)
			show_scores_and_update();
			playing = 0;
			XFlush(XtDisplay(canvas));
			return;
		}
		calculate();
		new_comb();
	}
	XFlush(XtDisplay(canvas));
	start_timer();
}


static XtIntervalId timer;

start_timer()
{
	unsigned long   interval;
	int             droprate;

	droprate = (int) (15000. / (start_level + 1. + ((float) score) / 1500));
	if (droprate < 0)
		interval = 0;
	else
		interval = droprate / 16;

	if (playing) {
		XFlush(XtDisplay(canvas));
		timer = XtAppAddTimeOut(context, interval, drop_block, NULL);
	}
}

stop_timer()
{
	if (timer) {
		XtRemoveTimeOut(timer);
		timer = NULL;
	}
}

init_all()
{
	int             i, j;
	XGCValues       gcv;
	Pixmap          boxstipple;

	for (i = 0; i < UWIDTH; i++)
		for (j = 0; j < UHEIGHT; j++)
			game_grid[i][j] = 0;

	srandom((int) time((time_t *) 0));

	XClearWindow(XtDisplay(canvas), XtWindow(canvas));
	boxstipple = XCreatePixmap(XtDisplay(canvas), XtWindow(canvas), BLCK_SIZE, BLCK_SIZE, 1);
	for (i = 0; i < 7; i++) {
		if (color_screen) {
		gcv.foreground = shape_table[i].foreground;
		gcv.background = shape_table[i].background;
		} else
		{
		gcv.foreground = WhitePixel(display, 0);
		gcv.background = BlackPixel(display, 0);
		}
		gcv.stipple = boxstipple;
		gcv.fill_style = FillSolid;
		BlcksGC[i] = XCreateGC(XtDisplay(canvas), XtWindow(canvas),
				     (unsigned long) GCForeground | GCBackground | GCStipple | GCFillStyle,
				     &gcv);
	}
	bit = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), bit_bits, bit_width, bit_height);
	DustPixmap = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), dust_bits, dust_width, dust_height);
	block = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), block_bits, block_width, block_height);
	ZetaPixmap = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), zet_bits, zet_width, zet_height);
	BombPixmap = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), bomb_bits, bomb_width, bomb_height);

	/*if(!color_screen) {*/
	letters[0] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), f_bits, f_width, f_height);
	letters[1] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), a_bits, a_width, a_height);
	letters[2] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), l_bits, l_width, l_height);
	letters[3] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), o_bits, o_width, o_height);
	letters[4] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), u_bits, u_width, u_height);
	letters[5] = XCreateBitmapFromData(XtDisplay(canvas), XtWindow(canvas), t_bits, t_width, t_height);
	/*}*/

	score = 0;
	visible_zeta = 0;
	Bombs_number = 0;
	visible_bomb = 0;
}


int
can_rot()
{
	int             i, j;
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;

	if (x == 0 || x == (UWIDTH - 1) || y == (UHEIGHT - 1))
		return 0;
	for (i = x - 1; i < x + 2; i++)
		for (j = y - 1; j < y + 2; j++)
			if (game_grid[i][j] != 0)
				return (0);
	return (1);
}

void
rot_proc()
{
	char            tmp;

	if (playing)
		if (can_rot()) {
			clear_shape();
			BlockOrient = !BlockOrient;
			if(BlockOrient == HORIZONTAL){
				tmp = BlockSet[0];
				BlockSet[0] = BlockSet[2];
				BlockSet[2] = tmp;
			}

			draw_shape();
		}
}

int             can_left();
void
left_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	if (playing)
		if (can_left()) {
			clear_shape();
			BlockXpos -= BLCK_SIZE;
			draw_shape();
			XFlush(XtDisplay(canvas));
		}
}

int             can_right();
void
right_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	if (playing)
		if (can_right()) {
			clear_shape();
			BlockXpos += BLCK_SIZE;
			draw_shape();
			XFlush(XtDisplay(canvas));
		}
}

void
drop_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	if (playing) {
		while (can_down()) {
			clear_shape();
			BlockYpos += BLCK_SIZE;
			draw_shape();
			XFlush(XtDisplay(canvas));
		}
		calculate();
		new_comb();
	}
}

new_comb()
{
	int             i;
	BlockXpos = (UWIDTH / 2) - 1;
	BlockXpos *= BLCK_SIZE;
	BlockYpos = BLCK_SIZE;
	BlockSet[0] = NextBlockSet[0];
        BlockSet[1] = NextBlockSet[1];
        BlockSet[2] = NextBlockSet[2];
	NextBlockSet[0] = random() % 6;
	NextBlockSet[1] = random() % 6;
	NextBlockSet[2] = random() % 6;
	BlockOrient =  HORIZONTAL;
	XSync(XtDisplay(canvas),True);
		show_next();
	visible_zeta = ((random() % 40) == 4);
	if (visible_zeta) {
		ZetaXpos = random() % UWIDTH;
		i = 0;
		while ((game_grid[ZetaXpos][i] == 0) && (i < UHEIGHT))
			i++;
		ZetaYpos = i - 1;
		draw_zeta();
	} else {
		visible_bomb = ((random() % 40) == 1);
		if (visible_bomb) {
			BombXpos = random() % UWIDTH;
			i = 0;
			while ((game_grid[BombXpos][i] == 0) && (i < UHEIGHT))
				i++;
			BombYpos = i - 1;
			draw_bomb();
		}
	}
}

clear_shape()
{
	if (BlockOrient == VERTICAL) {
		XClearArea(XtDisplay(canvas), XtWindow(canvas), BlockXpos, BlockYpos - BLCK_SIZE, BLCK_SIZE, BLCK_SIZE * 3, FALSE);
	} else {
		XClearArea(XtDisplay(canvas), XtWindow(canvas), BlockXpos - BLCK_SIZE, BlockYpos, BLCK_SIZE * 3, BLCK_SIZE, FALSE);
	}
}

draw_shape()
{
int             x = BlockXpos / BLCK_SIZE;
int             y = BlockYpos / BLCK_SIZE;
	if (BlockOrient == VERTICAL) {
		draw_bit(BlockSet[0],x,y-1);
		draw_bit(BlockSet[1],x,y);
		draw_bit(BlockSet[2],x,y+1);
	} else {
		draw_bit(BlockSet[0],x-1,y);
		draw_bit(BlockSet[1],x,y);
		draw_bit(BlockSet[2],x+1,y);
	}
	XFlush(XtDisplay(canvas));
}

draw_bomb()
{
	if (visible_bomb)
		XCopyPlane(XtDisplay(canvas), BombPixmap, XtWindow(canvas), BlcksGC[6], 
			   0, 0, BLCK_SIZE, BLCK_SIZE, BombXpos * BLCK_SIZE, BombYpos * BLCK_SIZE, 1L);
}

clear_bomb()
{
	if(visible_bomb) {
	clear_bit(BombXpos,BombYpos);
	visible_bomb = 0;
	}

}

draw_zeta()
{
	if (visible_zeta)
		XCopyPlane(XtDisplay(canvas), ZetaPixmap, XtWindow(canvas), BlcksGC[6], 0, 0, BLCK_SIZE, BLCK_SIZE, ZetaXpos * BLCK_SIZE, ZetaYpos * BLCK_SIZE, 1L);
}

clear_zeta()
{
	if(visible_zeta){
	clear_bit(ZetaXpos,ZetaYpos);
	visible_zeta =0;
	}

}

int
can_right()
{
	int             i, j;
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;

	if (BlockOrient == VERTICAL) {
		if (x == (UWIDTH - 1) || game_grid[x + 1][y + 1] != 0)
			return (0);
	} else {
		if (x == (UWIDTH - 2) || game_grid[x + 2][y] != 0)
			return (0);
	}
	return (1);
}
int
can_left()
{
	int             i, j;
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;

	if (BlockOrient == VERTICAL) {
		if (x == 0 || game_grid[x - 1][y + 1] != 0)
			return (0);
	} else {
		if (x == 1 || game_grid[x - 2][y] != 0)
			return (0);
	}
	return (1);
}

int
can_down()
{
	int             i, j;
	int             x = BlockXpos / BLCK_SIZE;
	int             y = BlockYpos / BLCK_SIZE;

	if (BlockOrient == VERTICAL) {
		if (y == (UHEIGHT - 2))
			return (0);
		if (game_grid[x][y + 2] != 0)
			return (0);
	} else {
		for (i = -1; i < 2; i++) {
			if (y == (UHEIGHT - 1))
				return (0);
			if (game_grid[x + i][y + 1] != 0)
				return (0);
		}
	}
	return (1);
}

int
find_shapes()
{
	int             i, j;
	int             res = 0;
	for (i = 0; i < UWIDTH; i++)
		for (j = 0; j < UHEIGHT; j++)
			tmp_grid[i][j] = 0;
	if (visible_zeta) {
		for (i = 0; i < UWIDTH; i++)
			for (j = 0; j < UHEIGHT; j++)
				if (game_grid[i][j] != 0) {
					tmp_grid[i][j] = ((random() % 5) == 4);
					if (tmp_grid[i][j]) {
						draw_dust(game_grid[i][j] - 1, i,j);
						removed = 1;
					}
				}
		visible_zeta = 0;
	}

	for (i = 0; i < UWIDTH; i++)
		for (j = 0; j < UHEIGHT; j++) {
			if (game_grid[i][j] != 0) {
				mark(game_grid[i][j], i, j, 0, 1, 0);
				mark(game_grid[i][j], i, j, 1, 0, 0);
				mark(game_grid[i][j], i, j, 1, 1, 0);
				mark(game_grid[i][j], i, j, 1, -1, 0);
			}
		}
}

int
mark(value, x, y, px, py, depth)
	int             value, x, y, px, py, depth;
{
	int             res;
	if (x < 0 || y < 0 || y == UHEIGHT || x == UWIDTH)
		return (0);
	if (game_grid[x][y] != value)
		return (0);

	if (depth > 1) {
		tmp_grid[x][y] = 1;
		removed = 1;
		draw_dust(game_grid[x][y] - 1, x,y);
		return (1 + mark(value, x + px, y + py, px, py, depth + 1));
	} else if (((res = mark(value, x + px, y + py, px, py, depth + 1)) + depth) > 1) {
		tmp_grid[x][y] = 1;
		removed = 1;
		draw_dust(game_grid[x][y] - 1, x,y);
		return (1 + res);
	}
}

void
circ_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	int             tmp;
	clear_shape();
	tmp = BlockSet[2];
	BlockSet[2] = BlockSet[1];
	BlockSet[1] = BlockSet[0];
	BlockSet[0] = tmp;
	draw_shape();
}

void
restore_widget(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{

	int             i, j;
	static int      shown=0;
        extern          show_logo();
	Display 	*display;

	display=XtDisplay(canvas);
        XClearWindow(display, XtWindow(canvas));


	for (i = 0; i < UWIDTH; i++)
		for (j = 0; j < UHEIGHT; j++)
			if (game_grid[i][j])
				draw_bit(game_grid[i][j] - 1,i,j);

	XDrawLine(display, XtWindow(canvas), lineGC, 
	UWIDTH*BLCK_SIZE+1, 0, UWIDTH*BLCK_SIZE+1, UHEIGHT* BLCK_SIZE);


#define WIDTH 600
#define HEIGHT (UHEIGHT*BLCK_SIZE)
	for (i = 301; i < WIDTH; i += block_width)
		for (j = 0; j < HEIGHT; j += block_height)
			XCopyPlane(display, block, XtWindow(canvas), lineGC,
				 0, 0, block_width, block_height, i, j, 1L);


	XDrawLine(display,XtWindow(canvas),thickGC,BOX2_X-6,BOX2_Y-3,BOX2_X+BOX2_WIDTH,BOX2_Y-3);
	XDrawLine(display,XtWindow(canvas),thickGC,BOX2_X-3,BOX2_Y,BOX2_X-3,BOX2_Y+BOX2_HEIGHT);

	XClearArea(display, XtWindow(canvas), BOX2_X, BOX2_Y,
		   BOX2_WIDTH, BOX2_HEIGHT, FALSE);
	print_string("XFALLOUT 1.0", BOX2_X + 15, BOX2_Y + 15);

	update_info();

	XDrawLine(display,XtWindow(canvas),thickGC,BOX1_X-6,BOX1_Y-3,BOX1_X+BOX1_WIDTH,BOX1_Y-3);
        XDrawLine(display,XtWindow(canvas),thickGC,BOX1_X-3,BOX1_Y,BOX1_X-3,BOX1_Y+BOX1_HEIGHT);

	XClearArea(display, XtWindow(canvas), BOX1_X, BOX1_Y,
		   BOX1_WIDTH, BOX1_HEIGHT, FALSE);
	print_string("Next:", BOX1_X+2,BOX1_Y+50);	
        if(!shown) {
		show_logo();
		if(usescorefile)
			read_scores_and_show();
		shown=1;
	}
	draw_zeta();
	draw_bomb();
	if (playing) {
		draw_shape();
	}
	show_next();
}

print_string(to_print, x, y)
	char           *to_print;
	int             x, y;
{
	static XFontStruct *t_font = NULL;
	static int      letter_width;
	static int      letter_height;

	XTextItem       text;

	if (t_font == NULL) {
		t_font = XQueryFont(display, XLoadFont(display, FONT_NAME));
		if (t_font == NULL) {
			fprintf(stderr,
			    "xvmines: Failed to load font %s\n", FONT_NAME);
			exit(1);
		}
		letter_width = t_font->max_bounds.rbearing - t_font->min_bounds.lbearing;
		letter_height = t_font->max_bounds.ascent + t_font->max_bounds.descent;
	}
	text.chars = to_print;
	text.nchars = strlen(to_print);
	text.delta = 0;
	text.font = t_font->fid;
	XDrawText(display, XtWindow(canvas), lineGC, x, y, &text, 1);
}

void
new_game_proc(w, event, pars, npars)
	Widget          w;
	XEvent         *event;
	String         *pars;
	Cardinal       *npars;
{
	int             i, j;

	if (!playing) {
		for (i = 0; i < UWIDTH; i++)
			for (j = 0; j < UHEIGHT; j++)
				game_grid[i][j] = 0;
		XClearArea(XtDisplay(canvas), XtWindow(canvas), 0, 0, 
			UWIDTH*BLCK_SIZE, UHEIGHT* BLCK_SIZE, FALSE);
		score = 0;
		visible_zeta = 0;
		visible_bomb = Bombs_number = 0;
		update_info();
		NextBlockSet[0] = random() % 6;
        	NextBlockSet[1] = random() % 6;
        	NextBlockSet[2] = random() % 6;
		playing = 1;
		new_comb();
		start_timer();
	}
}

add_bomb()
{
	Bombs_number++;
	visible_bomb = 0;
	update_info();
}

void
activate_bomb_proc(w, event, pars, npars)
        Widget          w;
        XEvent         *event;
        String         *pars;
        Cardinal       *npars;
{
	int             i, j;
        int             res = 0;
	if (playing && Bombs_number){
		removed =0;
        for (i = 0; i < UWIDTH; i++)
                for (j = 0; j < UHEIGHT; j++)
                        tmp_grid[i][j] = 0;
        for (i = 0; i < UWIDTH; i++)
                for (j = 0; j < UHEIGHT; j++)
                        if (game_grid[i][j] != 0) {
                         tmp_grid[i][j] = ((random() % 5) == 4);
                         if (tmp_grid[i][j]) {
			   draw_dust(game_grid[i][j] - 1,i,j);
                           removed = 1;
                         }
                        }
		Bombs_number -= 1;
		clear_bomb();	
		clear_zeta();
		update_info();
	while(removed)  {
		XFlush(XtDisplay(canvas));
		usleep(500000);
		if (removed) {
		 update_positions();
		}
		update_info();
		removed = 0;
		find_shapes();
	}
	}
}

update_info()
{
char message_str[20];
                XClearArea(XtDisplay(canvas), XtWindow(canvas),
                           BOX2_X, BOX2_Y + 35, 180, 110, FALSE);
 		sprintf(message_str, "Score: %d", score);
                print_string(message_str, BOX2_X + 5, BOX2_Y + 60);
                sprintf(message_str, "Level: %d", start_level + (int) (((float) score) / 1500.));
                print_string(message_str, BOX2_X + 5, BOX2_Y + 90);
                sprintf(message_str, "Bombs: %d", Bombs_number);
                print_string(message_str, BOX2_X + 5, BOX2_Y + 120);
}

show_next()
{
	if (playing && next) {
		draw_bit(NextBlockSet[0],14,3);
		draw_bit(NextBlockSet[1],15,3);
		draw_bit(NextBlockSet[2],16,3);
	}
}

clear_next()
{
	clear_bit(14,3)	;
	clear_bit(15,3)	;
	clear_bit(16,3)	;
}

void next_proc(w, event, pars, npars)
        Widget          w;
        XEvent         *event;
        String         *pars;
        Cardinal       *npars;
{
	next = !next;

	if(next)
		show_next();
	else
		clear_next();
}


void increase_level_proc(w, event, pars, npars)
        Widget          w;
        XEvent         *event;
        String         *pars;
        Cardinal       *npars;
{
start_level ++;
if(start_level > 15)
	start_level = 15;
update_info();
if(playing) {
	stop_timer();
	start_timer();
}
}

void decrease_level_proc(w, event, pars, npars)
        Widget          w;
        XEvent         *event;
        String         *pars;
        Cardinal       *npars;
{
if(!playing && start_level) {
start_level --;
update_info();
}
}

draw_bit(i,x,y)
int i,x,y;
{
if (color_screen)
  XCopyPlane(XtDisplay(canvas), letters[i], XtWindow(canvas), BlcksGC[i], 0, 0, BLCK_SIZE, BLCK_SIZE, 
		x * BLCK_SIZE, y * BLCK_SIZE, 1L);
else
 XCopyPlane(XtDisplay(canvas), letters[i], XtWindow(canvas), BlcksGC[i], 0, 0, BLCK_SIZE, BLCK_SIZE, 
		x * BLCK_SIZE, y * BLCK_SIZE, 1L);
}

clear_bit(x,y)
	int x,y;
{
XClearArea(XtDisplay(canvas), XtWindow(canvas), x * BLCK_SIZE, 
	y * BLCK_SIZE, BLCK_SIZE, BLCK_SIZE, FALSE);
}

draw_dust(i,x,y)
int x,y;
{
XCopyPlane(XtDisplay(canvas), DustPixmap, XtWindow(canvas), BlcksGC[i], 
	  0, 0, BLCK_SIZE, BLCK_SIZE, x * BLCK_SIZE, y * BLCK_SIZE, 1L);
}


finish()
{
	int i,j;

	for(j=0;j<UHEIGHT;j++) {
	 for(i=0;i<UWIDTH;i++) {
	  draw_bit(i%6,i,j);
  	  game_grid[i][j]=(i%6)+1;
	  }
	XFlush(XtDisplay(canvas));
	usleep(100000);
	}
}


